from models import * 
import scheme_var

import numpy as np
import matplotlib.pyplot as plt
from scipy import signal
from scipy.fft import fft, fftfreq
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.constants import ACTIVE
import torch
import torch.nn as nn
import mne

def new_window(Title, size):
    newindow= tk.Toplevel()
    newindow.geometry(size)
    newindow.title(Title)
    newindow.wm_attributes('-topmost', True)
    return newindow

def get_weight(filepath):
    global conv1, conv2
    net= SCCNet()
    weights= torch.load(filepath, map_location= torch.device('cpu'))
#     for weight in weights:
#         if 'weight' in weight:
#             print(f'{weight}: {weights[weight].shape}')

    #spatial ((kernelsize),n_kernel,1)
    conv1= weights['conv1.weight'].detach().cpu().numpy().squeeze()

    #temporal (n_kernel, inputnode  ,kernelsize)
    conv2= weights['conv2.weight'].detach().cpu().numpy().squeeze()
    conv2= conv2.reshape(20*22,12) 
    
    return conv1,conv2

def montage_select(w_load):
    w = new_window('montage selection', '300x200')
    stdmontagelist = ttk.Combobox(w,
                    values=["BCI_IV 2a",  # only for test 
                            "standard_1005", 
                            "standard_1020",
                            "standard_1020_32ch",
                            "standard_alphabetic",
                            "standard_postfixed",
                            "standard_prefixed",
                            "standard_primed",
                            "biosemi16",
                            "biosemi32",
                            "biosemi64",
                            "biosemi128",
                            "biosemi256",
                            "easycap-M1",
                            "easycap-M10",
                            "EGI_256",
                            "GSN-HydroCel-32",
                            "GSN-HydroCel-64_1.0",
                            "GSN-HydroCel-65_1.0",
                            "GSN-HydroCel-128",
                            "GSN-HydroCel-129",
                            "GSN-HydroCel-256",
                            "GSN-HydroCel-257",
                            "mgh60",
                            "mgh70",
                            "artinis-octamon",
                            "artinis-brite23"
                            ],
                    state="readonly"
            )
    
  
    Confirm = ttk.Button(w ,text="Confirm", command=(lambda:[channel_select(w,w_load, stdmontagelist)]))
    stdmontagelist.pack()
    Confirm.pack()



def channel_select(w,w_load, stdmontagelist):
    name = stdmontagelist.get()
    print(name)
    if name== 'BCI_IV 2a':
        montage= mne.channels.make_standard_montage("standard_1020")
        chs=['Fz', 'FC3', 'FC1','FCz', 'FC2','FC4',
             'C5','C3','C1','Cz','C2','C4','C6',
             'CP3','CP1','CPz','CP2','CP4',
             'P1','Pz','P2','POz']
          
    elif name== 'standard_1020_32ch':
        montage= mne.channels.make_standard_montage("standard_1020")
        chs= ['Fp1', 'Fp2', 'F7', 'F3', 'Fz', 'F4', 'F8',
              'A1','T3','C3','Cz','C4', 'T4', 'A2',
              'T5', 'P3', 'Pz','P4','T6','O1','O2']
    else:
        montage= mne.channels.make_standard_montage(name)
        chs = list(montage.get_positions()['ch_pos'].keys())
        print(chs)

    w.destroy()

    w2 = new_window('channel selection', '300x300')
    chlist = tk.Listbox(w2, selectmode=tk.EXTENDED)
    for c in chs:
        chlist.insert(tk.END, c)


    #Confirm = tk.Button(w2 ,text="Confirm", command=(lambda:[Spatial_kernel(w2, montage, chlist,  conv1)]))
    Confirm = ttk.Button(w2 ,text="Confirm", command=(lambda:select_ch(w2, w_load, name, chlist,montage)))

    chlist.pack()
    Confirm.pack()


def select_ch(w2, w_load, name, chlist,montage):
    
    ch_select = chlist.curselection()
    electrode = []
    for c in ch_select:
        electrode.append(chlist.get(c))
    print(electrode)

    scheme_var.montage = montage
    scheme_var.electrode= electrode
    tk.Label(w_load, text= name).grid(row=8, column=1)
    w2.destroy()

def Spatial_kernel(montage, electrode, conv1):
    
    #w.destroy()
     
    montage= montage.get_positions()['ch_pos']
    position= [montage[i] for i in electrode]
    position= np.asarray(position)
    pos_count = len(position)


    fig, axes = plt.subplots(5, int(np.ceil(pos_count/5)), figsize=(10, 10))
    for i in range(int(5*np.ceil(pos_count/5))):
        if i < pos_count:
            print(conv1[:i].squeeze().shape)
            mne.viz.plot_topomap(data=conv1[:,i].squeeze(),
                                pos=position[:,0:2],
                                axes= axes[int(i/5), i%5],
                                show=False)
        else:
            fig.delaxes(axes[int(i/5), i%5])
    
    plt.show()
# def Spatial_kernel(conv1):
#     #conv1,_= get_weight(filepath)
#     electrode=['Fz', 'FC3', 'FC1','FCz', 'FC2','FC4',
#            'C5','C3','C1','Cz','C2','C4','C6',
#           'CP3','CP1','CPz','CP2','CP4',
#           'P1','Pz','P2','POz']

#     montage= mne.channels.make_standard_montage('standard_1020')
#     montage= montage.get_positions()['ch_pos']

#     position= [montage[i] for i in electrode]
#     position= np.asarray(position)
    
#     fig, ax = plt.subplots(5, 5, figsize=(5, 5))
#     ax[4, -3].axis('off'), ax[4, -2].axis('off'), ax[4, -1].axis('off')
    
#     #for row in range(5):
#         #for col in range(5):
#     for i in range(22):
#         mne.viz.plot_topomap(data=conv1[i],
#                                      pos=position[:,0:2],
#                                      axes= ax[int(i/5), int(i%5)],
#                                      show=False)
           
#     plt.show()            
                
def plot_mag(conv2):
    N = 12
    T = 1.0 / 128.0
    
    freq = np.unique(np.round(abs(fftfreq(N, d=T)),1)) 
    y = conv2                                                 
    y_pad = np.pad(y, pad_width = [(0,0),(0,1200-12)], mode='constant') # (440, 1100) 
    y_pad = np.log10(abs(fft(y_pad)))  
    y_pad = y_pad[y_pad.argmax(1).argsort()]                 
    y_pad = y_pad[:,0:int(y_pad.shape[1]/2)]                 

    fig= plt.figure()
    subplot = fig.add_subplot(111)
    subplot.set_xlabel('Frequency (Hz)')
    subplot.set_ylabel('Kernel (Spatio-Temporal)')
    subplot.set_title(f'Magnitude response')
    ticks_location = [0, 100,200,300,400,500,599]
    subplot.set_xticks(ticks_location)
    subplot.set_xticklabels(freq)
    
    img= subplot.imshow(y_pad, vmin= None, vmax= None)
    fig.colorbar(img)
    fig.tight_layout()
    
    plt.show()

    return 


import numpy as np

# x,y from mne input -> use xy_data function retrun correct format xy data -> main( xy data )
def confirm_xy_size(x_train, y_train, x_test, y_test): #numpy array (n_trials, n_chs, n_times) (n_trials,1)
    #xy 
    if not ((len(x_train.shape)==3) and (len(y_train.shape)==2)) and ((len(x_test.shape)==3) and (len(y_test.shape)==2)):
        raise ValueError ('shape of x or y is not correct.')

    if not (x_train.shape[0]== y_train.shape[0]) and (x_test.shape[0]== y_test.shape[0]) :
        raise ValueError (f'numbers of trials of x and y have to be the same length.')
    
    #train test 
    text=['trials', 'channels', 'timepoints']
    for i, j in enumerate(text): 
        if not (x_train.shape[i]== x_test.shape[i]) :
         raise ValueError (f'Numbers of {j} of x and y have to be the same length.')

    return x_train, y_train, x_test, y_test


import torch
import torch.optim as optim
import torch.nn as nn

import numpy as np
import tkinter as tk
from tkinter import ttk
import time
            
            
def Test():

    weightfolder= scheme_var.weightfolder
    Testloader= scheme_var.testloader
    net= scheme_var.net

    net.load_state_dict(torch.load(f'{weightfolder}/temp.pth'))
    net.eval()

    correct=0
    
#     classes = ('left_arm', 'right_arm', 'leg', 'tongue')
#     class_correct = list(0. for i in range(4))
#     class_total = list(0. for i in range(4))
    
    #yt_ =[]
    ypred=np.zeros((288))
    ypr=np.zeros((288,scheme_var.n_class))
    softmax = nn.Softmax(dim=1)
    flag=0

    for xt,yt in Testloader:
        with torch.no_grad():
            output= net(xt)
            nor_y= softmax(output).reshape(-1)
            #print(nor_y[:,nor_y.argmax()])
            if output.argmax() == yt:
                correct+=1

            #yt_.append( yt.detach().cpu().numpy())
            ypred[flag]= nor_y.argmax().detach().cpu().numpy()
            ypr[flag]= nor_y.detach().cpu().numpy()
            flag+=1

        # acc for each class
#         c = (yt_pred == yt)
#         for i in range(yt.size(0)):
#             label = yt[i]
#             class_correct[label] += c[i].item()
#             class_total[label] += 1

    acc= correct/ len(Testloader)
    

    print('=================')
    print(f' Acc: {acc:.4f}')
    print('=================')
    
    #yt_= np.asarray(yt_)

#     for i in range(4):
#         print(f'{classes[i]} : {(class_correct[i] / class_total[i]):.4f}')
#     print('=================')

    return acc, ypred, ypr

#=======================main=================================
import variable
import scheme_var

def Scheme():
    epochs= scheme_var.epochs
    trainloader= scheme_var.trainloader
    Valloader= scheme_var.valloader
    weightfolder= scheme_var.weightfolder
    netname= scheme_var.netname
    savedweight= scheme_var.savedweight
    n_cuda= scheme_var.n_cuda


   
    dev = torch.device(f"cuda:{n_cuda}" if torch.cuda.is_available() else "cpu")
    net= netname.to(dev)
    #FT mode
    if savedweight !=0 : 
        print('aaaaaaaaaaaaaaaaa ft')
        net.load_state_dict(torch.load(savedweight))
        
    criterion = nn.CrossEntropyLoss() 
    opt = optim.Adam(net.parameters(),lr= scheme_var.lr)
    
    min_loss=1e10
    for epoch in range(1,epochs+1):
        if scheme_var.stop_thread== False:
            print(scheme_var.stop_thread)
            scheme_var.net= net #update net for testing 
            variable.update_epo= epoch
            time.sleep(0.1) 

            tloss= 0
            for x, y in trainloader:
                net.train()
                y_pred= net(x)
                tloss_ = criterion(y_pred, y)
                tloss+= tloss_
                tloss_.backward()
                opt.step()
                opt.zero_grad()

            net.eval()   
            vloss=0
            for xv, yv in Valloader:
                with torch.no_grad():
                    yv_pred= net(xv)
                    vloss_ = criterion(yv_pred, yv)
                    vloss += vloss_.item()
                    
            if vloss< min_loss:
                min_loss= vloss
                torch.save(net.state_dict(), f'{weightfolder}/temp.pth')
                
            if (epoch + 1) % 10 == 0:
                print(f"epoch:{epoch+1}  tloss:{tloss:.4f} vloss:{vloss:.4f} minloss:{min_loss:.4f}")

        else:
                break
    

    # if scheme_var.stop_thread == False:
    #     acc_,y_true , scheme_var.ypred, scheme_var.ypr = Test(weightfolder, testloader, net= net )
    #     scheme_var.acc.set(f'Accuracy: {acc_:4f}')
        
    


        



import torch
import torch.utils.data as Data
import scipy.io
import scheme_var


    
def Individual_Dataset(X_train, Y_train,X_test, Y_test, FT= False):

    bs= scheme_var.bs
    ratio= scheme_var.val_ratio
    n_cuda= scheme_var.n_cuda 

    dev= torch.device(f"cuda:{n_cuda}") if torch.cuda.is_available() else torch.device("cpu")
    
    
    # data1 = scipy.io.loadmat(X_train)
    # data2 = scipy.io.loadmat(Y_train)
    # data3 = scipy.io.loadmat(X_test)
    # data4 = scipy.io.loadmat(Y_test)


    # x_T= torch.Tensor(data1['x_train']).unsqueeze(1)
    # y_T= torch.Tensor(data2['y_train']).view(-1)
    # x_test= torch.Tensor(data3['x_test']).unsqueeze(1)
    # y_test= torch.Tensor(data4['y_test']).view(-1)
    
    x_T= torch.Tensor(X_train).unsqueeze(1)
    y_T= torch.Tensor(Y_train).view(-1)
    x_test= torch.Tensor(X_test).unsqueeze(1)
    y_test= torch.Tensor(Y_test).view(-1)

    x_train, y_train, x_val, y_val= train_val_split(x_T, y_T, ratio)


    
    x_train= x_train.to(dev)
    y_train= y_train.long().to(dev)
    x_val= x_val.to(dev)
    y_val= y_val.long().to(dev)
    x_test= x_test.to(dev)
    y_test= y_test.long().to(dev)
    
        
    print('x_train shape:', x_train.shape)
    print('y_train shape:', y_train.shape)
    print('x_val shape:', x_val.shape)
    print('y_val shape:', y_val.shape)
    print('x_test shape:', x_test.shape)
    print('y_test shape:', y_test.shape)
       
    Train_Dataset = Data.TensorDataset(x_train, y_train )
    Val_Dataset = Data.TensorDataset(x_val, y_val )
    Test_Dataset = Data.TensorDataset(x_test, y_test )
    
    
    Trainloader = Data.DataLoader(Train_Dataset, 
                                  batch_size= bs,
                                  shuffle= True,
                                  num_workers = 0)

    Valloader = Data.DataLoader(Val_Dataset, 
                                  batch_size= len(y_val),
                                  shuffle= False,
                                  num_workers = 0)
    
    Testloader = Data.DataLoader(Test_Dataset, 
                                  batch_size= 1,
                                  shuffle= False,
                                  num_workers = 0)

                            
    return Trainloader,  Valloader, Testloader,  x_train.shape, x_val.shape,  x_test.shape
   


def train_val_split(x_T, y_T, ratio):
    s = y_T.argsort() ## 排序並分開四個class
    x_T = x_T[s]
    y_T = y_T[s]

    cL = y_T[y_T==0].size()[0]

    class1_x = x_T[:cL]
    class2_x = x_T[cL:cL*2]
    class3_x = x_T[cL*2:cL*3]
    class4_x = x_T[cL*3:cL*4]

    class1_y = y_T[:cL]
    class2_y = y_T[cL:cL*2]
    class3_y = y_T[cL*2:cL*3]
    class4_y = y_T[cL*3:cL*4]

    vL= int(len(class1_y )*ratio) #1/32*4=1/8
 

    x_train = torch.cat((class1_x[:-vL], class2_x[:-vL], class3_x[:-vL], class4_x[:-vL]))
    y_train = torch.cat((class1_y[:-vL], class2_y[:-vL], class3_y[:-vL], class4_y[:-vL]))

    x_val = torch.cat((class1_x[-vL:], class2_x[-vL:], class3_x[-vL:], class4_x[-vL:]))
    y_val = torch.cat((class1_y[-vL:], class2_y[-vL:], class3_y[-vL:], class4_y[-vL:]))

    return x_train, y_train, x_val, y_val


    
def Ind_withoutval(trainpath, testpath, bs):

    dev= torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")

    T = scipy.io.loadmat(trainpath)
    E = scipy.io.loadmat(testpath)

    x_train= torch.Tensor(T['x_train'])
    y_train= torch.Tensor(T['y_train']).view(-1)
    x_train= x_train.unsqueeze(1).to(dev)
    y_train= y_train.view(-1).long().to(dev)
    x_test= torch.Tensor(E['x_test']).unsqueeze(1).to(dev)
    y_test= torch.Tensor(E['y_test']).view(-1).long().to(dev)
    
    Train_Dataset = Data.TensorDataset(x_train, y_train )
    Test_Dataset = Data.TensorDataset(x_test, y_test )
    
    
    Trainloader = Data.DataLoader(dataset= Train_Dataset, 
                                  batch_size= bs,
                                  shuffle= True)
    
    Testloader = Data.DataLoader(dataset= Test_Dataset, 
                                  batch_size= 1,
                                  shuffle= False)
                                 
    return Trainloader,  Testloader, x_train.shape, x_test.shape



global w_bar, weightfolder, savedweight, trainloader, valloader, testloader
global sf, ch, tp, net,epochs, lr, sub, netname, stop_thread, val_ratio, n_cuda, n_class, bs
global ypred, ypr, acc
global montage, electrode
global x_train, y_train, x_test , y_test 

x_train=0
y_train=0
x_test=0
y_test=0 

w_bar= 0
weightfolder= 0
savedweight=0 

trainloader= 0
valloader =0
testloader= 0
val_ratio=0

n_cuda=0
n_class=0
epochs=0
lr=0
sub=0
netname=0 
stop_thread= 0
ypred= 0
ypr= 0
acc=0
net=0
sf=0
ch=0
tp=0
bs=0

montage=0
electrode=0


from GUI import main
from datautil import xy

import mne
import numpy as np 
import scipy.io

if __name__ == '__main__':
    x_train= scipy.io.loadmat('G:\我的雲端硬碟/109_2 course/Project/visSCC/GUI/xydata/xtrain.mat')
    y_train= scipy.io.loadmat('G:\我的雲端硬碟/109_2 course/Project/visSCC/GUI/xydata/ytrain.mat')
    x_test= scipy.io.loadmat('G:\我的雲端硬碟/109_2 course/Project/visSCC/GUI/xydata/xtest.mat')
    y_test= scipy.io.loadmat('G:\我的雲端硬碟/109_2 course/Project/visSCC/GUI/xydata/ytest.mat')
    
    x_train= x_train['x_train']
    y_train= y_train['y_train']
    x_test= x_test['x_test']
    y_test= y_test['y_test']
    # epochs = mne.io.read_epochs_eeglab('G:/我的雲端硬碟/109_2 course/BCI/lab3/Day 2_SSVEP.set')
    # x_train=epochs._data
    # y_train = np.ones((100,1))
    # print(x_train.shape, y_train.shape)
    # x_test=epochs._data
    # y_test = np.ones((100,1))

    X_train, Y_train, X_test, Y_test = xy.confirm_xy_size(x_train, y_train, x_test, y_test)

    main(X_train, Y_train, X_test, Y_test)  



import setuptools
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"),"r") as fh:
    long_description = fh.read()



setuptools.setup(
     name='vissccnet',  
     version='0.2.4',  
     author="Ya-Lin Huang", 
     author_email="bai06hua28lin.bt06@nctu.edu.tw",
     description="A CNN model training and weight visualization GUI",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/skywalkerylh/visSCCNet",
     install_requires=['mne', 'numpy', 'scipy', 'matplotlib'],
     packages=setuptools.find_packages(), #Use for other package dependencies.
     classifiers=[
        #'Development Status :: 3 - Alpha',  #not sure 

        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        'Topic :: Software Development :: Build Tools',

        "Topic :: Scientific/Engineering :: Artificial Intelligence",

        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.7',
    ],
    keywords='eeg deep-learning brain-state-decoding',
    include_package_data=False, # 沒有其他打包文件
    zip_safe=False, #whether the package is installed in compressed mode or regular mode.
)


import tkinter as tk
from tkinter.constants import CENTER
from tkinter.ttk import *
from tkinter import messagebox

import threading
import numpy as np
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
import scipy.io

from freq_response import *
from dataloader_gui import Individual_Dataset
from train_test import Test, Scheme
import variable
import scheme_var

#====================== Main========================
#--------------------------Control Center----------------------------------
def main(X_train, Y_train, X_test, Y_test):

    global win_main #new
    win_main = tk.Tk()
    win_main.geometry('400x400')
    win_main.title("CNN Weight Visualization")
    win_main.configure(background='White')

    # Control Center
    tk.Label(win_main, text="X_train  :　", bg= 'White').grid(row=0, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="X_test :　", bg= 'White').grid(row=1, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Trainsize :　", bg= 'White').grid(row=2, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Validsize :　", bg= 'White').grid(row=3, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Testsize :　", bg= 'White').grid(row=4, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Sampliing rate :　", bg= 'White').grid(row=5, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Model :　", bg= 'White').grid(row=6, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Epoch :　", bg= 'White').grid(row=7, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Lr :　", bg= 'White').grid(row=8, column=0, ipady=7, padx=20)
    tk.Label(win_main, text="Weight :　", bg= 'White').grid(row=9, column=0, ipady=7, padx=20)

    #create menu
    menubar = tk.Menu(win_main)

    # Menu- File
    filemenu = tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label='File', menu=filemenu)
    filemenu.add_command(label='Load Dataset ', command=lambda:Load_dataset(X_train, Y_train, X_test, Y_test))
    filemenu.add_command(label='Load model ', command=lambda:load_model_struct())
    filemenu.add_command(label='Load weight ', command=Weightname)

    # Menu-Model
    Model = tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label='Model', menu=Model)
    Model.add_command(label='Training setting', command=lambda:Model_Preparation())
    Model.add_command(label='Start Training', command=(lambda:bar()))
    #Model.add_command(label='Result', command=(lambda:Result()))


    #Menu- Plot
    Plot = tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label='Plot', menu= Plot)
    Plot.add_command(label='Spatial kernel ', command=lambda:Spatial_kernel(scheme_var.montage,scheme_var.electrode, conv1))
    Plot.add_command(label='Magnitude response ', command=lambda:plot_mag(conv2))


    win_main.config(menu=menubar)
    win_main.mainloop()


# Menu- File- Load dataset=================================    
def Load_dataset(X_train, Y_train, X_test, Y_test):

    w=new_window(Title='Load Dataset', size='400x400')
    #topic
    # tk.Label(w, text="x_train :　", bg= 'White').grid(row=0, column=0)
    # tk.Label(w, text="y_train :　", bg= 'White').grid(row=1, column=0)
    # tk.Label(w, text="x_test :　", bg= 'White').grid(row=2, column=0)
    # tk.Label(w, text="y_test :　", bg= 'White').grid(row=3, column=0)
    tk.Label(w, text="Validation ratio :　", bg= 'White').grid(row=4,padx=15)
    tk.Label(w, text="Batchsize :　", bg= 'White').grid(row=5,padx=15)
    tk.Label(w, text="Cuda device :　", bg= 'White').grid(row=6,padx=15)
    tk.Label(w, text="Number of Classes :　", bg= 'White').grid(row=7,padx=15)
    tk.Label(w, text="EEG Montage :　", bg= 'White').grid(row=8,padx=15)
    tk.Label(w, text="Sampling rate :　", bg= 'White').grid(row=9,padx=15)
    tk.Label(w, text="Number of channel :　", bg= 'White').grid(row=10,padx=15)

    #textinput
    # x_train = tk.Entry(w)
    # y_train = tk.Entry(w)
    # x_test = tk.Entry(w)
    # y_test = tk.Entry(w)
    Valratio = tk.Entry(w)
    Batchsize = tk.Entry(w)
    n_cuda = tk.Entry(w)
    n_class = tk.Entry(w)
    sf = tk.Entry(w)
    ch = tk.Entry(w)

    # x_train.grid(row=0, column=1)
    # y_train.grid(row=1, column=1)
    # x_test.grid(row=2, column=1)
    # y_test.grid(row=3, column=1)
    Valratio.grid(row=4, column=1)
    Batchsize.grid(row=5, column=1)
    n_cuda.grid(row=6, column=1)
    n_class.grid(row=7, column=1)
    sf.grid(row=9, column=1)
    ch.grid(row=10, column=1)

    #load train test path by bottom
    # ttk.Button(w, text="File", command=(lambda:path2entry(x_train, 'x_train')), width=10).grid(row=0,column=2)
    # ttk.Button(w, text="File", command=(lambda:path2entry(y_train,'y_train')), width=10).grid(row=1,column=2)
    # ttk.Button(w, text="File", command=(lambda:path2entry(x_test,'x_test')), width=10).grid(row=2,column=2)
    # ttk.Button(w, text="File", command=(lambda:path2entry(y_test, 'y_test')), width=10).grid(row=3,column=2)
    ttk.Button(w, text="Select ", command=(lambda:montage_select(w)), width=10).grid(row=8,column=2)


    Confirm = ttk.Button(w ,text="Confirm", 
                         command=(lambda:show_input(w,Batchsize, Valratio, n_cuda, n_class, sf, ch, X_train, Y_train, X_test, Y_test)), width=10).grid(row=11, column=1, pady=5)

# def path2entry(entry, set_type):
#     path = tk.filedialog.askopenfilename()
#     entry.insert(tk.END, path)
#     data= read_path(path)
#     identify_path_type(path, set_type) #identify path 
#     identify_set_type(data, set_type) #identify data 


# def read_path(path):
#     try: 
#         if '.mat' in path:
#             path = scipy.io.loadmat(path)
#             settype=['x_train', 'y_train', 'x_test', 'y_test']
#             for atype in settype:
#                 if atype in path.keys():
#                     data= path[atype]

#         if '.set' in path: 
#             data = mne.io.read_epochs_eeglab(path)
#             data =data._data
            
#         if '.edf' in path: 
#             data= mne.io.read_raw_edf(path,preload= True)
#             data =data._data
            
#         return data     

#     except TypeError:
#         print('The file format is not supported.')

# def identify_path_type(path, set_type):#辨別 xt, yt, xtest, ytest path

#     global X_trainpath, Y_trainpath, X_testpath, Y_testpath

#     if set_type=='x_train':
#         X_trainpath= path
#     elif set_type=='y_train':
#         Y_trainpath= path
#     elif set_type=='x_test':
#         X_testpath= path 
#     elif set_type=='y_test':
#         Y_testpath= path
        
# def identify_set_type(data, set_type):#辨別 xt, yt, xtest, ytest data

#     global X_train, Y_train, X_test, Y_test

#     if set_type=='x_train':
#         X_train= data
#     elif set_type=='y_train':
#         Y_train= data
#     elif set_type=='x_test':
#         X_test= data 
#     elif set_type=='y_test':
#         Y_test= data



def show_input(w, Batchsize, Valratio, n_cuda, n_class, sf ,ch,X_train, Y_train, X_test, Y_test):

    scheme_var.val_ratio= float(Valratio.get())
    scheme_var.bs= int(Batchsize.get())
    scheme_var.n_cuda= int(n_cuda.get())
    scheme_var.n_class= int(n_class.get())
    scheme_var.sf= int(sf.get())
    scheme_var.ch= int(ch.get())

    w.destroy()


    # tk.Label(win_main, text= "/".join(X_trainpath.split('/')[-2:])).grid(row=0, column=1 )
    # tk.Label(win_main, text= "/".join(X_testpath.split('/')[-2:])).grid(row=1, column=1)


    scheme_var.trainloader, scheme_var.valloader,  scheme_var.testloader, trainsize, validsize,  testsize= Individual_Dataset(X_train, Y_train,X_test, Y_test)
                                                                                                                            
    tk.Label(win_main, text= trainsize).grid(row=2, column=1 )
    tk.Label(win_main, text= validsize).grid(row=3, column=1 )
    tk.Label(win_main, text= testsize).grid(row=4, column=1)
    tk.Label(win_main, text= scheme_var.sf).grid(row=5, column=1)
    scheme_var.tp= trainsize[-1]

    print('x_train size: ',X_train.shape,
            '\ny_train size: ', Y_train.shape,
            '\nx_test size: ', X_test.shape,
            '\ny_test size: ', Y_test.shape)

    # print('x_train path: ',X_trainpath,
    #         '\ny_train path: ', Y_trainpath,
    #         '\nx_test path: ', X_testpath,
    #         '\ny_test path: ', Y_testpath)


# Menu- File- Load Model=================================    
def load_model_struct(): #load .py with models structure
    win_lms=new_window(Title='Load model structure', size='600x400')
    modelist = ttk.Combobox(win_lms,
                    values=['SCCNet',
                            'EEGNet',
                            'ShallowConvNet',
                            ],
                    state="readonly"
            )

    Confirm = ttk.Button(win_lms ,text="Confirm", command=(lambda:[model_select(win_lms, modelist)]))
    modelist.pack()
    Confirm.pack()

def model_select(win_lms, modelist):
    exec(open('models.py').read(), globals()) #put all variable to globals()
    name = modelist.get()
    print(name)
    scheme_var.netname= eval(str(name+'()'))
    tk.Label(win_main, text=name).grid(row=6, column=1) #show on center 

    win_lms.destroy()

# Menu- File- Load weight====================================
# def Filename(): #revise
#     name= tk.filedialog.askopenfilename(initialdir = "/")
#     FileName = tk.Label(win_main, text= name)
#     FileName.place(x=90, y=20)

def Weightname():
    global conv1, conv2

    file= tk.filedialog.askopenfilename()
    shortpath= "/".join(file.split('/')[-2:])
    tk.Label(win_main, text= shortpath).grid(row=9, column=1)
    conv1,conv2= get_weight(filepath=file)
    scheme_var.savedweight= file
#print(f'conv1: {conv1.shape}')
#conv1size = tk.Label(window, text= f'conv1: {conv1.shape}').grid(row=9, column=1)

#conv2size = tk.Label(window, text= f'conv2: {conv2.shape}').grid(row=10, column=1)

# Menu- Model- Set hyperparameter =================================    

def Model_Preparation():
    win_p=new_window(Title='Training setting ', size='400x400')
    #topic
    tk.Label(win_p, text="Epoch :　").grid(row=0, column=0, ipady=10, padx=20)
    tk.Label(win_p, text="lr :　").grid(row=1, column=0, ipady=10, padx=20)

    tk.Label(win_p, text="save weight : 　").grid(row=2,column=0, ipady=10, padx=20)

    Epoch = tk.Entry(win_p)
    Lr = tk.Entry(win_p)

    Save_weight = tk.Entry(win_p)

    Epoch.grid(row=0, column=1)
    Lr.grid(row=1, column=1)
    Save_weight.grid(row=2, column=1)

    weightfolder= ttk.Button(win_p, text="Folder", command=(lambda:folder2entry(Save_weight)), width=10).grid(row=2,column=2)

    Confirm = ttk.Button(win_p ,text="Confirm", command=(lambda:show_para(win_p, Epoch, Lr)), width=10).grid(row=3, column=1)


def show_para(win_p, Epoch, Lr ):

#global epochs, lr
    scheme_var.epochs= int(Epoch.get())
    scheme_var.lr= float(Lr.get())


    #close w
    win_p.destroy()
    #para in control display 
    tk.Label(win_main, text= scheme_var.epochs).grid(row=7, column=1 )
    tk.Label(win_main, text= scheme_var.lr).grid(row=8, column=1)
    tk.Label(win_main, text= shortweightfolder).grid(row=9, column=1)


# Menu- Model- Start training =================================

def bar(): #w_bar, ypred, ypr remove from scheme_var? 
    global progressbar, value_label, buttom 
    scheme_var.w_bar = new_window(Title='Evaluation', size='350x150')

    progressbar = Progressbar(scheme_var.w_bar, mode='determinate',length= 350, maximum= scheme_var.epochs+1)
    progressbar.pack(anchor= tk.CENTER)

    value_label = tk.Label(scheme_var.w_bar, text= update_progress_label())
    value_label.pack(anchor= tk.CENTER)

    scheme_var.acc= tk.StringVar()
    scheme_var.acc.set('Accuracy: None')
    tk.Label(scheme_var.w_bar, textvariable= scheme_var.acc).pack(anchor= tk.CENTER)


    buttom= ttk.Button(scheme_var.w_bar, text="Start Training",command=lambda:start_train_thread(test_buttom))
    buttom.pack(side= tk.LEFT, padx= 20, pady=10)


    test_buttom= ttk.Button(scheme_var.w_bar, text="Start Testing",command=lambda:start_test_thread(test_buttom, label_buttom))
    test_buttom.pack(side= tk.LEFT, padx= 10, pady=10)
    test_buttom['state']= tk.DISABLED

    label_buttom= ttk.Button(scheme_var.w_bar, text="Predictive Label",command=lambda:PredictiveLabel(scheme_var.ypr, scheme_var.ypred ))
    label_buttom.pack(side= tk.LEFT, padx=10,pady=10)
    label_buttom['state']= tk.DISABLED

    scheme_var.w_bar.protocol("WM_DELETE_WINDOW", lambda: quit_train()) #interaction between app and window manaager

def start_test_thread(test_buttom, label_buttom):
    test_buttom['state']= tk.NORMAL
    acc,scheme_var.ypred, scheme_var.ypr = Test() 
    scheme_var.acc.set(f'Accuracy: {acc:.4f}')
    label_buttom['state']= tk.NORMAL


def start_train_thread(test_buttom):
    global train_thread

    scheme_var.stop_thread= False
    buttom['state'] = tk.DISABLED #disable buttom while training 
    train_thread= threading.Thread(target= Scheme, daemon= True) #aquire epoch from train scheme
    progressbar.start(interval= 400)
    train_thread.start()
    scheme_var.w_bar.after(400, check_train_thread)
    test_buttom['state']= tk.NORMAL

def check_train_thread():
    print(variable.update_epo)

    progressbar["value"]= variable.update_epo
    value_label["text"] = update_progress_label()

    if train_thread.is_alive() :
        scheme_var.w_bar.after(400, check_train_thread) # control update epoch speed
    else: 
        progressbar.stop()
        

def update_progress_label():
    label= progressbar["value"]
    return f"Epoch: {label}/ {scheme_var.epochs}"

def quit_train():
    if messagebox.askokcancel('Quit', 'Do you want to quit?'):
        scheme_var.stop_thread= True
        scheme_var.w_bar.destroy()
        

#display the predictive label and probability 
def PredictiveLabel( y_pr, y_pred):
    w= new_window('Predictive Label', '550x250')
    s = ttk.Style()
    # Create style used by default for all Frames
    s.configure('TFrame', background='green')
    frame= ttk.Frame(w,  height=1, width=550, style= 'TFrame')
    ttk.Button(frame, text="Save", command=lambda:save_y(y_pr, y_pred)).pack(fill='x')

    table= ttk.Treeview(w)
    #scroll bar 
    sb = Scrollbar(w, orient=tk.VERTICAL)
    sb.pack(side=tk.RIGHT, fill=tk.Y)
    table.config(yscrollcommand=sb.set)
    sb.config(command=table.yview)

    table['column']= ['ID','Predictive Label']+[f'Class {i}'for i in range(scheme_var.n_class)]
    table.column('#0', width=0, stretch=tk.NO)
    table.column('ID', anchor=CENTER, width=30 )
    table.column('Predictive Label', anchor=CENTER, width=130)
    [table.column(f'Class {i}', anchor=tk.CENTER, width=80) for i in range(scheme_var.n_class)]

    table.heading('#0', text='', anchor=CENTER)
    table.heading('ID', text='ID', anchor=CENTER)
    table.heading('Predictive Label', text='Predictive Label', anchor=CENTER)
    [table.heading(f'Class {i}', text=f'Class {i}', anchor=tk.CENTER) for i in range(scheme_var.n_class)]

    for row in range(len(y_pred)):
        table.insert(parent='', index=row, iid= row, text='', values=[f'{row}', y_pred[row]]+[y_pr[row,i] for i in range(scheme_var.n_class)])
    table.pack()     

def save_y(y_pr, y_pred):
    folder= tk.filedialog.askdirectory()
    y_pred= y_pred.reshape(-1,1)
    print('Probability size:', y_pr.shape,'\nLabel size', y_pred.shape)
    np.savetxt(folder+'/data.csv', np.concatenate((y_pred, y_pr), axis=1), delimiter=',')


# Menu- Plot- spatial ==============================================    
# plot_spatial, mag_response in freq_response.py


#====================== others========================
def new_window(Title, size):
    newindow= tk.Toplevel()
    newindow.geometry(size)
    newindow.title(Title)
    newindow.configure(background='White')
    newindow.wm_attributes('-topmost', True)
    return newindow


def folder2entry(entry):
    global shortweightfolder
    folder= tk.filedialog.askdirectory()
    shortweightfolder= "/".join(folder.split('/')[-2:])
    entry.insert(tk.END, folder)
    scheme_var.weightfolder= folder    





import torch
import torch.nn as nn
import math
import scheme_var


class EEGNet(nn.Module):
    def __init__(self):
        super(EEGNet, self).__init__()

        self.tp= scheme_var.tp
        self.ch= scheme_var.ch
        self.sf= scheme_var.sf
        self.n_class=scheme_var.n_class
        self.half_sf= math.floor(self.sf/2)
        print(f'timepoint:{self.tp}',
              f'\nch:{self.ch}',
              f'\nsf:{self.sf}',
              f'\nclasses:{self.n_class}'
            )

        self.F1 = 8
        self.F2 = 16
        self.D = 2 #spatial filters 
        
        self.conv1 = nn.Sequential(  
        #temporal kernel size(1, floor(sf*0.5)) means 500ms EEG at sf/2
        #padding=(0, floor(sf*0.5)/2) maintain raw data shape 
            nn.Conv2d(1, self.F1, (1, int(self.half_sf)), padding=(0, int(self.half_sf/2)), bias=False),
            nn.BatchNorm2d(self.F1)
        )

        self.conv2 = nn.Sequential(       
            # spatial kernel size (n_ch, 1)
            nn.Conv2d(self.F1, self.D*self.F1, (self.ch, 1), groups=self.F1, bias=False),
            nn.BatchNorm2d(self.D*self.F1),
            nn.ELU(),
            nn.AvgPool2d((1, 4)), #reduce the sf to sf/4
            nn.Dropout(0.5) # 0.25 in cross-subject classification beacuse the training size are larger 
        )

        self.Conv3 = nn.Sequential(
        # kernel size=(1, floor((sf/4))*0.5) means 500ms EEG at sf/4 Hz 
            nn.Conv2d(self.D*self.F1, self.D*self.F1, (1, int(self.half_sf/4)), padding=(0, int(self.half_sf/8)), groups=self.D*self.F1, bias=False),
            nn.Conv2d(self.D*self.F1, self.F2, (1, 1), bias=False),
            nn.BatchNorm2d(self.F2),
            nn.ELU(),
            nn.AvgPool2d((1, 8)), #dim reduction
            nn.Dropout(0.5)
        )
        
        #(floor((sf/4))/2 * timepoint//32, n_class)
        self.classifier = nn.Linear(int(self.half_sf/4)* int(self.tp//32), self.n_class, bias=True)
       
    def forward(self, x):
        x = self.conv1(x)
        #print(x.shape)
        x = self.conv2(x)
        #print(x.shape)
        x = self.Conv3(x)
        #print(x.shape)
        #(-1, sf/8* timepoint//32)
        x = x.view(-1, int(self.half_sf/4)* int(self.tp//32))
        #print(x.shape)
        x = self.classifier(x)
       
        return x

class SCCNet(nn.Module):
    def __init__(self):
        super(SCCNet, self).__init__() # input:bs, 1, channel, sample

        self.tp= scheme_var.tp
        self.ch= scheme_var.ch
        self.sf= scheme_var.sf
        self.n_class=scheme_var.n_class
        self.octsf= math.floor(self.sf*0.1)

        print(f'timepoint:{self.tp}',
        f'\nch:{self.ch}',
        f'\nsf:{self.sf}',
        f'\nclasses:{self.n_class}')


        # (1, n_ch, kernelsize=(n_ch,1))
        self.conv1 = nn.Conv2d(1, 22, (self.ch, 1))  
        self.Bn1 = nn.BatchNorm2d(22) #(n_ch) 
        #kernelsize=(1, floor(sf*0.1)) padding= (0, floor(sf*0.1)/2)
        self.conv2 = nn.Conv2d(22, 20, (1, int(self.octsf)), padding=(0, int(self.octsf/2))) 
        self.Bn2   = nn.BatchNorm2d(20)
        
        self.Drop1 = nn.Dropout(0.5) 
        #kernelsize=(1, sf/2) revise to 128/2?  stride=(1, floor(sf*0.1))
        self.AvgPool1 = nn.AvgPool2d((1, int(self.sf/2)), stride=(1, int(self.octsf))) 
        # (20* ceiling((timepoint-sf/2)/floor(sf*0.1)), n_class)
        self.classifier = nn.Linear(20*int(math.ceil((self.tp- self.sf/2)/ self.octsf)), self.n_class, bias=True)
        

    def forward(self, x):
        x = self.conv1(x) #(128,22,1,562)
        #print(x.shape)
        x = self.Bn1(x) 
        x = self.conv2(x) #(128,20,1,563)
        #print(x.shape)
        x = self.Bn2(x)
        x = x ** 2
        x = self.Drop1(x)
        x = self.AvgPool1(x) #(128,20,1,42)
        #print(x.shape)
        x = torch.log(x) 
        x = x.view(-1, int(20*math.ceil((self.tp- self.sf/2)/ self.octsf)))
        x = self.classifier(x)

       
        return x


class ShallowConvNet(nn.Module):
    def __init__(self):
        super(ShallowConvNet, self).__init__()

        self.tp= scheme_var.tp
        self.ch= scheme_var.ch
        self.sf= scheme_var.sf
        self.n_class=scheme_var.n_class
        self.octsf= int(math.ceil(self.sf*0.1)) #13
        self.tpconv1= int(self.tp- self.octsf+1) #550 time point after conv1 
        self.apstride= int(math.ceil(self.octsf/2))
 
        print(f'timepoint:{self.tp}',
              f'\nch:{self.ch}',
              f'\nsf:{self.sf}',
              f'\nclasses:{self.n_class}')
        
        # kernel size=(ceil(sf*0.1)
        self.conv1 = nn.Conv2d(1, 40, (1, self.octsf), bias=False)
        #(n_ch,1)
        self.conv2 = nn.Conv2d(40, 40, (self.ch, 1), bias=False)
        self.Bn1   = nn.BatchNorm2d(40)
       # not sure 
        self.AvgPool1 = nn.AvgPool2d((1, int(self.apstride*5)), stride=(1,self.apstride ))
       
        self.Drop1 = nn.Dropout(0.25)
        self.classifier = nn.Linear(int(40*math.ceil((self.tpconv1-self.apstride*5) / self.apstride)), 
                                    self.n_class, bias=True)
        

    def forward(self, x):
        x = self.conv1(x)
        #print('conv1',x.shape) # 36,40,22,550(no padding)
        x = self.conv2(x)
        #print('conv2',x.shape) # 36,40,1,550 
        x = self.Bn1(x)
        #print('bn1',x.shape) 
        x = x ** 2
        x = self.AvgPool1(x)
        #print('avp',x.shape) # 36,40,1,74 
        x = torch.log(x)
        x = self.Drop1(x)
        #print('drop1',x.shape)
        x = x.view(-1, int(40*math.ceil((self.tpconv1-self.apstride*5 )/ self.apstride))) #40*74
        x = self.classifier(x)

        #x = self.softmax(x)
        return x